unit cDIBImage;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: cDIBImage.PAS, released August 28, 2000.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2000 Peter Morris.
All Rights Reserved.

Purpose of file:
This is a DIB version of TImage.  Has opacity / angle etc.

Contributor(s):
None as yet


Last Modified: August 28, 2000
Current Version: 1.00

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org
or at http://www.stuckindoors.com/dib


Known Issues:
To be updated !
-----------------------------------------------------------------------------}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  cDIBControl, cDIB, cDIBImageList;

type
  TCustomDIBImage = class(TCustomDIBControl)
  private
    { Private declarations }
    FAngle          : TAngle;
    FAutoSize       : Boolean;
    FCenter         : Boolean;
    FIndexMain      : TDIBImageLink;
    FScale          : Real;
    FStretch        : Boolean;
    procedure SetAutosize(const Value: Boolean);
    procedure SetAngle(const Value: TAngle);
    procedure SetScale(const Value: Real);
    procedure SetCenter(const Value: Boolean);
    procedure SetStretch(const Value: Boolean);
  protected
    { Protected declarations }
    procedure ImageChanged(Index : Integer; Operation: TDIBOperation); override;
    procedure UpdateImage; virtual;

    property Angle            : TAngle
      read FAngle
      write SetAngle;
    property AutoSize         : Boolean
      read FAutoSize
      write SetAutosize;
    property Center           : Boolean
      read FCenter
      write SetCenter;
    property IndexMain       : TDIBImageLink
      read FIndexMain
      write FIndexMain;
    property Scale            : Real
      read FScale
      write SetScale;
    property Stretch          : Boolean
      read FStretch
      write SetStretch;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure Paint; override;
  published
    { Published declarations }
  end;

  TDIBImage = class(TCustomDIBImage)
  private
  protected
  public
  published
    property Accelerator;
    property Align;
    property Anchors;
    property Angle;
    property AutoSize;
    property Center;
    property Children;
    property Constraints;
    property Cursor;
    property DIBFeatures;
    property DIBImageList;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Hint;
    property IndexMain;
    property Opacity;
    property ParentShowHint;
    property PopupMenu;
    property Scale;
    property ShowHint;
    property Stretch;
    property DIBTabOrder;
    property Tag;
    property Visible;

    {$I WinControlEvents.inc}
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property OnResize;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DIB', [TDIBImage]);
end;

{ TCustomDIBImage }

constructor TCustomDIBImage.Create(AOwner: TComponent);
begin
  inherited;
  FAutosize := False;
  FScale := 100;
  AddIndexProperty(FIndexMain);
  AddTemplateProperty('Angle');
  AddTemplateProperty('AutoSize');
  AddTemplateProperty('Center');
  AddTemplateProperty('Opacity');
  AddTemplateProperty('Scale');
  AddTemplateProperty('Stretch');
end;

destructor TCustomDIBImage.Destroy;
begin
  inherited;
end;

procedure TCustomDIBImage.UpdateImage;
var
  TheDIB                      : TMemoryDIB;
  Sizes                       : TPoint;
begin
  if AutoSize and (IndexMain.DIBIndex >= 0) then begin
    if IndexMain.GetImage(TheDIB) then begin
      Sizes := GetRotatedSize(TheDIB.Width,TheDIB.Height,Angle,Scale,Scale);
      if (Sizes.x > 0) and (Sizes.Y > 0) then SetBounds(Left,Top,Sizes.x,Sizes.Y);
    end;
  end;
  Invalidate;
end;

{
procedure TCustomDIBImage.ImageChanged(Sender: TObject);
var
  NewSize           : TPoint;
begin
  if Sender = FImage then begin
    if AutoSize then begin
      NewSize := GetRotatedSize(Image.Width,Image.Height,Image.Angle,Image.Scale);
      SetBounds(Left,Top,NewSize.X,NewSize.Y);
    end;
    Invalidate;
  end;
end;
}

procedure TCustomDIBImage.ImageChanged(Index : Integer; Operation: TDIBOperation);
begin
  if (Index = IndexMain.DIBIndex) then
    UpdateImage;
end;

procedure TCustomDIBImage.Paint;
var
  TheDIB                      : TMemoryDIB;
  XPos,
  YPos                        : Integer;
  NewSizes                    : TPoint;
begin
  inherited;
  if IndexMain.GetImage(TheDIB) then begin
    TheDIB.AutoSize := True;
    TheDIB.Angle := Angle;
    TheDIB.Scale := Scale;
    XPos := 0;
    YPos := 0;

    if Stretch then begin
      TheDIB.ScaleX := Width * 100 div TheDIB.Width;
      TheDIB.ScaleY := Height * 100 div TheDIB.Height
    end;

    NewSizes := GetRotatedSize(TheDIB.Width, TheDIB.Height, Angle,
      TheDIB.ScaleX , TheDIB.ScaleY);

    if Center and not AutoSize then begin
      XPos := Width div 2 - (NewSizes.X div 2);
      YPos := Height div 2 - (NewSizes.Y div 2);
    end;
    TheDIB.Draw(XPos,YPos,TheDIB.Width,TheDIB.Height,ControlDIB,0,0);
  end;
end;

procedure TCustomDIBImage.SetAngle(const Value: TAngle);
begin
  FAngle := Value;
  UpdateImage;
end;

procedure TCustomDIBImage.SetAutosize(const Value: Boolean);
begin
  if Value = AutoSize then exit;
  FAutosize := Value;
  if Value then begin
    FStretch := False;
    UpdateImage;
  end;
end;

procedure TCustomDIBImage.SetScale(const Value: Real);
begin
  FScale := Value;
  Stretch := False;
  UpdateImage;
end;

procedure TCustomDIBImage.SetCenter(const Value: Boolean);
begin
  if Value = Center then exit;
  FCenter := Value;
  UpdateImage;
end;

procedure TCustomDIBImage.SetStretch(const Value: Boolean);
begin
  FStretch := Value;
  if Value then AutoSize := False;
  Invalidate;
end;

end.
